# An all-in-one cut-down version of GHC's fptools/mk directory.
#
# HOpenGL - a binding of OpenGL and GLUT for Haskell.
# Copyright (c) 2002, Sven Panne
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# 3. Neither the name of the author nor the names of its contributors may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#
# Typical usage in a Makefile:
#
# TOP     = .
# OPT     = -O
# HC_OPTS = -fglasgow-exts -H30M
# HS_PROG = FooBar
# include $(TOP)/Makefile.mk


#------------------------------------------------------------------------------
# 		GNU Standard targets
#
# `all'
#      Compile the entire program. This should be the default target.
#      This target need not rebuild any documentation files; Info files
#      should normally be included in the distribution, and DVI files
#      should be made only when explicitly asked for.
# 
# `install'
#      Compile the program and copy the executables, libraries, and so on
#      to the file names where they should reside for actual use. If
#      there is a simple test to verify that a program is properly
#      installed, this target should run that test.
# 
# `uninstall'
#      Delete all the installed files that the `install' target would
#      create (but not the noninstalled files such as `make all' would
#      create).
# 
# `clean'
#      Delete all files from the current directory that are normally
#      created by building the program.  Don't delete the files that
#      record the configuration. Also preserve files that could be made
#      by building, but normally aren't because the distribution comes
#      with them.
# 
#      Delete `.dvi' files here if they are not part of the
#      distribution.
# 
# `distclean'
#      Delete all files from the current directory that are created by
#      configuring or building the program. If you have unpacked the
#      source and built the program without creating any other files,
#      `make distclean' should leave only the files that were in the
#      distribution.
# 
# `mostlyclean'
#      Like `clean', but may refrain from deleting a few files that
#      people normally don't want to recompile. For example, the
#      `mostlyclean' target for GCC does not delete `libgcc.a', because
#      recompiling it is rarely necessary and takes a lot of time.
# 
# `maintainer-clean'
#      Delete everything from the current directory that can be
#      reconstructed with this Makefile.  This typically includes
#      everything deleted by distclean , plus more: C source files
#      produced by Bison, tags tables, Info files, and so on.
# 
#      One exception, however: `make maintainer-clean' should not delete
#      `configure' even if `configure' can be remade using a rule in the
#      Makefile. More generally, `make maintainer-clean' should not delete
#      anything that needs to exist in order to run `configure' and then
#      begin to build the program.
# 
# `TAGS'
#      Update a tags table for this program.
# 
# `info'
#      Generate any Info files needed.
# 
# `dvi' `ps'
#      Generate DVI/PS files for all TeXinfo documentation.
# 
# `dist'
#      Create a distribution tar file for this program. The tar file
#      should be set up so that the file names in the tar file start with
#      a subdirectory name which is the name of the package it is a
#      distribution for. This name can include the version number.
# 
# `check'
#      Perform self-tests (if any). The user must build the program
#      before running the tests, but need not install the program; you
#      should write the self-tests so that they work when the program is
#      built but not installed.

#------------------------------------------------------------------------------
# This rule makes sure that "all" is the default target, regardless of where
# it appears. THIS RULE MUST REMAIN FIRST!

.PHONY: default

default: all

#------------------------------------------------------------------------------

-include $(TOP)/config.mk

#------------------------------------------------------------------------------
# Here are the diabolically clever rules that propagates "make <t>" to
# directories in SUBDIRS for each "recursive target" <t>
#
# Note about recursively invoking make: we'd like make to drop all the
# way back to the top level if it fails in any of the
# sub(sub-...)directories.  This is done by setting the -e flag to the
# shell during the loop, which causes an immediate failure if any of
# the shell commands fail.

# One exception: if the user gave the -i or -k flag to make in the
# first place, we'd like to reverse this behaviour.  So we check for
# these flags, and set the -e flag appropriately.  NOTE: watch out for
# the --no-print-directory flag which is passed to recursive
# invocations of make.
#
# NOTE: Truly weird use of exit below to stop the for loop dead in
# its tracks should any of the sub-makes fail. By my reckoning, 
#  "cmd || exit $?" should be equivalent to "cmd"

ifneq "$(SUBDIRS)" ""
all TAGS mostlyclean clean distclean maintainer-clean install info depend ::
	@echo "------------------------------------------------------------------------"
	@echo "Recursively making \`$@' in $(SUBDIRS) ..."
	@echo "PWD = $(shell pwd)"
	@echo "------------------------------------------------------------------------"
# Don't rely on -e working, instead we check exit return codes from sub-makes.
	@case '${MFLAGS}' in *-[ik]*) x_on_err=0;; *-r*[ik]*) x_on_err=0;; *) x_on_err=1;; esac; \
	for i in $(SUBDIRS); do \
	  echo "------------------------------------------------------------------------"; \
	  echo "$(MAKE) $@ $(MFLAGS);"; \
	  echo "in $(shell pwd)/$$i"; \
	  echo "------------------------------------------------------------------------"; \
	  $(MAKE) --no-print-directory -C $$i $(MFLAGS) $@; \
	  if [ $$? -eq 0 -o $$x_on_err -eq 0 ] ;  then true; else exit 1; fi; \
	done
	@echo "------------------------------------------------------------------------"
	@echo "Finished making \`$@' in $(SUBDIRS) ..."
	@echo "PWD = $(shell pwd)"
	@echo "------------------------------------------------------------------------"
endif

# Selectively building subdirectories.
ifneq "$(SUBDIRS)" ""
$(SUBDIRS) ::
	  $(MAKE) -C $@ $(MFLAGS)
endif

#------------------------------------------------------------------------------
# Paths and standard options

ifeq "$(HC)" ""
HC              = ghc
endif
OPT             += #-O
# to split object files, add -split-objs -odir $*_split
HC_OPTS         += $(OPT) $($*_HC_OPTS)

ifeq "$(MKDEPENDHS)" ""
MKDEPENDHS      = $(HC)
endif

ifeq "$(MKDEPENDC)" ""
MKDEPENDC       = gcc -MM
endif
# SUP: One of my greatest and most evil hacks...
MKDEPENDC_OPTS  += -I$(shell cut -c2- "$(shell which ghc)" | grep "^libdir='" | sed "s/libdir='\(.*\)'.*/\1/")/includes

ifeq "$(LX)" ""
LX              = alex
endif
LX_OPTS         +=

ifeq "$(HAPPY)" ""
HAPPY           = happy
endif
HAPPY_OPTS      +=

ifeq "$(GREENCARD)" ""
GREENCARD       = green-card
endif
GREENCARD_OPTS  +=

ifeq "$(RM)" ""
RM              = rm -f
endif

ifeq "$(FIND)" ""
FIND            = find
endif

ifeq "$(AR)" ""
AR              = ar
endif
ifeq "$(AR_OPTS)" ""
AR_OPTS         = cqs
endif

ifeq "$(RANLIB)" ""
RANLIB          = @:
endif

ifeq "$(HSTAGS)" ""
HSTAGS          = hstags
endif

ifeq "$(ETAGS)" ""
ETAGS          = etags
endif

ifeq "$(XARGS)" ""
XARGS          = xargs
endif

#------------------------------------------------------------------------------
# All files with a known suffix are considered as sources. The ones which are
# not should be included in EXCLUDE_SRCS.

ALL_SRCS       += $(wildcard *.lx *.x *.ly *.y *.gc *.lhs *.hs *.c)
SRCS            = $(filter-out $(EXCLUDE_SRCS),$(ALL_SRCS)) $(EXTRA_SRCS)

LX_SRCS         = $(filter %.lx %.x,$(SRCS))
HAPPY_SRCS      = $(filter %.ly %.y,$(SRCS))
GREENCARD_SRCS  = $(filter %.gc,$(SRCS))
GENERATED_HS    = $(addsuffix .hs,$(basename $(LX_SRCS) $(HAPPY_SRCS) $(GREENCARD_SRCS)))

# make sure the generated files are included only once in the list of haskell sources
HS_SRCS         = $(filter %.lhs %.hs,$(filter-out $(GENERATED_HS),$(SRCS)) $(GENERATED_HS))
HS_IFACES       = $(addsuffix .hi,$(basename $(HS_SRCS)))
HS_HCS          = $(addsuffix .hc,$(basename $(HS_SRCS)))
HS_OBJS         = $(addsuffix .o,$(basename $(HS_SRCS)))

C_SRCS          = $(filter %.c,$(filter-out $(EXTRA_SRCS),$(SRCS))) $(filter %.c,$(EXTRA_SRCS))
C_OBJS          = $(addsuffix .o,$(basename $(C_SRCS)))

MKDEPENDHS_SRCS = $(HS_SRCS)
# remove GHC-generated FFI stubs
MKDEPENDC_SRCS  = $(filter-out %_stub.c,$(C_SRCS))

TAGS_HS_SRCS    = $(HS_SRCS)
TAGS_C_SRCS     = $(C_SRCS)

#------------------------------------------------------------------------------
# For each of these variables that is defined we generate one "all" rule and
# one rule for the variable itself:
#	HS_PROG		Haskell program
#	C_PROG		C program
#	LIBRARY		Library

.PHONY: all

# Haskell programs
ifneq "$(HS_PROG)" ""
all :: $(HS_PROG)
$(HS_PROG) :: $(HS_OBJS) $(C_OBJS)
	$(HC) -o $@ $(HC_OPTS) $(LD_OPTS) $(HS_OBJS) $(C_OBJS) $(LIBS)
endif

# C programs
ifneq "$(C_PROG)" ""
all :: $(C_PROG)
$(C_PROG) :: $(C_OBJS)
	$(CC) -o $@ $(CC_OPTS) $(LD_OPTS) $(C_OBJS) $(LIBS)
endif

# Libraries/archives
ifneq "$(LIBRARY)" ""
all :: $(LIBRARY)

ifeq "$(LIB_OBJS)" ""
LIB_OBJS = $(HS_OBJS) $(C_OBJS)
endif

define BUILD_LIB
$(RM) $@
$(AR) $(AR_OPTS) $@ $(LIB_OBJS) $(STUB_OBJS)
$(RANLIB) $@
endef

# For Haskell object files, we might have chosen to split up the object files.
# Test for whether the library being built is consisting of Haskell files by
# (hackily) checking whether HS_SRCS is empty or not.
ifneq "$(HS_SRCS)" ""
ifneq "$(filter -split-objs,$(HC_OPTS))" ""
define BUILD_LIB
$(RM) $@
$(FIND) $(patsubst %.o,%_split,$(LIB_OBJS)) $(STUB_OBJS) -name '*.o' -print 2> /dev/null| $(XARGS) $(XARGS_OPTS) ar q $@
$(RANLIB) $@
endef
endif
endif

$(LIBRARY) :: $(LIB_OBJS)
	$(BUILD_LIB)
endif

#------------------------------------------------------------------------------

.PHONY: install uninstall

#------------------------------------------------------------------------------
# Cleaning up
#
# The following three variables are used to control
# what gets removed when doing `make clean' 
#
# MOSTLYCLEAN_FILES   object code etc., but not stuff
#                     that is slow to recompile and/or stable
#
# CLEAN_FILES  all files that are created by running make.
#
# MAINTAINER_CLEAN_FILES also clean out machine-generated files
#                        that may require extra tools to create.
#
MOSTLY_CLEAN_FILES     += $(HS_OBJS) $(C_OBJS) *_stub.c *_stub.h *_stub_ffi.h
CLEAN_FILES            += $(C_PROG) $(HS_PROG) $(LIBRARY) $(HS_IFACES) $(HS_HCS)
MAINTAINER_CLEAN_FILES += $(GENERATED_HS)

# `Standard' set of files to clean out.
MOSTLY_CLEAN_FILES     += *~ *.CKP *.ln *.BAK *.bak *.o *core a.out errs ,*   \
                          *.a .emacs_* tags TAGS *.ind *.ilg *.idx *.idx-prev \
                          *.aux *.aux-prev *.dvi *.log *.toc *.lot *.lof      \
                          *.blg *.info *.itxi *.itex *.ihtml *.cb .depend.bak

# Hack for Haskell files different from Main.[l]hs without "module Main"
CLEAN_FILES            += .depend Main.hi

DIST_CLEAN_FILES       += config.cache config.log config.status \
                          $(filter-out configure,$(basename $(wildcard *.in)))


.PHONY: realclean mostlyclean clean distclean maintainer-clean

# If (Haskell) object files are split, cleaning up  consist of descending
# into the directories where the myriads of object files have been put.
define REMOVE_SPLITTED
:
endef

ifneq "$(HS_SRCS)" ""
ifneq "$(filter -split-objs,$(HC_OPTS))" ""
define REMOVE_SPLITTED
$(FIND) $(patsubst %.o,%_split,$(HS_OBJS) $(C_OBJS)) -name '*.o' -print 2> /dev/null | $(XARGS) $(XARGS_OPTS) $(RM) __rm_food
rmdir $(patsubst %.o,%_split,$(HS_OBJS) $(C_OBJS)) > /dev/null 2>&1 || true
endef
endif
endif

# realclean is just a synonym for maintainer-clean
realclean: maintainer-clean

# NOTE: The dependecies are expanded out here, because otherwise there
# is an ugly repetition of deletions in recursive makes.
mostlyclean::
	$(REMOVE_SPLITTED)
	$(RM) __rm_food $(MOSTLY_CLEAN_FILES)

clean::
	$(REMOVE_SPLITTED)
	$(RM) __rm_food $(MOSTLY_CLEAN_FILES) $(CLEAN_FILES)

distclean::
	$(REMOVE_SPLITTED)
	$(RM) __rm_food $(MOSTLY_CLEAN_FILES) $(CLEAN_FILES) $(DIST_CLEAN_FILES)

maintainer-clean::
	@echo 'This command is intended for maintainers to use; it'
	@echo 'deletes files that may need special tools to rebuild.'
	$(REMOVE_SPLITTED)
	$(RM) __rm_food $(MOSTLY_CLEAN_FILES) $(CLEAN_FILES) $(DIST_CLEAN_FILES) $(MAINTAINER_CLEAN_FILES)

#------------------------------------------------------------------------------
# Tags

.PHONY: TAGS tags

tags TAGS:: $(TAGS_HS_SRCS) $(TAGS_C_SRCS)
	@$(RM) TAGS
	@touch TAGS
ifneq "$(TAGS_HS_SRCS)" ""
	$(HSTAGS) $(HC_OPTS) $(HSTAGS_OPTS) -- $(TAGS_HS_SRCS)
endif
ifneq "$(TAGS_C_SRCS)" ""
	$(ETAGS) -a $(TAGS_C_SRCS)
endif

#------------------------------------------------------------------------------
# Makefile debugging: to see the effective value used for a Makefile variable
# VAR, do   make show VALUE=VAR

.PHONY: show

show:
	@echo '$(VALUE)=$($(VALUE))'

#--------------------------------------------------------------------------

.PHONY: dvi ps

#--------------------------------------------------------------------------

.PHONY: dist

#--------------------------------------------------------------------------

.PHONY: check

#--------------------------------------------------------------------------
# Haskell Suffix Rules

HC_SPLIT_PRE  = $(RM) $@ ; if [ ! -d $(basename $@)_split ]; then mkdir $(basename $@)_split; else \
 $(FIND) $(basename $@)_split -name '*.o' -print | $(XARGS) $(XARGS_OPTS) $(RM) __rm_food ; fi
HC_SPLIT_POST = touch $@

HC_PRE_OPTS   = $(patsubst %,$(HC_SPLIT_PRE) ; ,$(filter -split-objs,$(HC_OPTS)))
HC_POST_OPTS  = $(patsubst %,$(HC_SPLIT_POST) ; ,$(filter -split-objs,$(HC_OPTS)))

%.o: %.hs
	$(HC_PRE_OPTS)
	$(HC) $(HC_OPTS) -c $< -o $@
	$(HC_POST_OPTS)

%.o: %.lhs	 
	$(HC_PRE_OPTS)
	$(HC) $(HC_OPTS) -c $< -o $@
	$(HC_POST_OPTS)

%_stub.c: %.o
	@:

# Here's an interesting rule! The .hi file depends on the .o file, so if the
# .hi file is dated earlier than the .o file (commonly the case, when
# interfaces are stable) this rule just makes sure that the .o file, is up to
# date.  Then it does nothing to generate the .hi file from the .o file,
# because the act of making sure the .o file is up to date also updates the .hi
# file (if necessary).

%.hi : %.o
	@if [ ! -f $@ ] ; then echo Panic! $< exists, but $@ does not. ; exit 1; fi

#-----------------------------------------------------------------------------
# Happy Suffix Rules

# ???
.PRECIOUS: %.hs

%.hs : %.ly
	$(HAPPY) $< $(HAPPY_OPTS)

%.hs : %.y
	$(HAPPY) $< $(HAPPY_OPTS)

#-----------------------------------------------------------------------------
# Lx Suffix Rules

%.hs : %.lx
	$(LX) $(LX_OPTS) $<

%.hs : %.x
	$(LX) $(LX_OPTS) $<

#-----------------------------------------------------------------------------
# Green-card Suffix Rules

%.hs : %.gc
	$(GREENCARD) $(GREENCARD_OPTS) $< -o $@

#-----------------------------------------------------------------------------
# C-related suffix rules

C_PRE_OPTS   = $(patsubst %,$(HC_SPLIT_PRE) ; ,$(filter -split-objs,$(HC_OPTS)))
C_POST_OPTS  = $(patsubst %,$(HC_SPLIT_POST) ; ,$(filter -split-objs,$(HC_OPTS)))

%.o : %.c
	$(C_PRE_OPTS)
	$(CC) $(CC_OPTS) -c $< -o $(basename $@)/$@
	$(C_POST_OPTS)

#-----------------------------------------------------------------------------
# The depend target has to cope with a set of files that may have different
# ways of computing their dependencies, i.e., a Haskell  module's dependencies
# are computed differently from C files.
#
# Note that we don't compute dependencies automatically, i.e., have the
# .depend file be a target that is dependent on the Haskell+C sources,
# and then have the `depend' target depend on `.depend'. The reason for
# this is that when GNU make is processing the `include .depend' statement
# it records .depend as being a Makefile. Before doing any other processing,
# `make' will try to check to see if the Makefiles are up-to-date. And,
# surprisingly enough, .depend has a rule for it, so if any of the source
# files change, it will be invoked, *regardless* of what target you're making.
#
# So, for now, the dependencies has to be re-computed manually via `make depend'
# whenever a module changes its set of imports. Doing what was outlined above
# is only a small optimisation anyway, it would avoid the recomputation of
# dependencies if the .depend file was newer than any of the source modules.
#
.PHONY: depend

depend :: $(MKDEPENDHS_SRCS) $(MKDEPENDC_SRCS)
	@$(RM) .depend
	@touch .depend
ifneq "$(MKDEPENDHS_SRCS)" ""
	$(MKDEPENDHS) -M -optdep-f -optdep.depend $(MKDEPENDHS_OPTS) $(patsubst -odir $*_split,,$(HC_OPTS)) $(MKDEPENDHS_SRCS)
endif
ifneq "$(MKDEPENDC_SRCS)" ""
# This is how it is done in the fptools, but we don't have mkdependC
#	$(MKDEPENDC) -f .depend $(MKDEPENDC_OPTS) -- $(CC_OPTS) -- $(MKDEPENDC_SRCS) 
	-$(MKDEPENDC) $(MKDEPENDC_OPTS) $(CC_OPTS) $(MKDEPENDC_SRCS) >> .depend
endif
# No dependencies? Remove .depend!
	@if test -f .depend -a ! -s .depend ; then $(RM) .depend ; fi

# the above patsubst is a hack to remove the '-odir $*_split' from HC_OPTS which is
# present when we're splitting objects.

-include .depend
