dnl HOpenGL - a binding of OpenGL and GLUT for Haskell.
dnl Copyright (c) 2002, Sven Panne
dnl All rights reserved.
dnl 
dnl Redistribution and use in source and binary forms, with or without
dnl modification, are permitted provided that the following conditions are met:
dnl 
dnl 1. Redistributions of source code must retain the above copyright notice,
dnl    this list of conditions and the following disclaimer.
dnl 
dnl 2. Redistributions in binary form must reproduce the above copyright
dnl    notice, this list of conditions and the following disclaimer in the
dnl    documentation and/or other materials provided with the distribution.
dnl 
dnl 3. Neither the name of the author nor the names of its contributors may be
dnl    used to endorse or promote products derived from this software without
dnl    specific prior written permission.
dnl 
dnl THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
dnl AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
dnl IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
dnl ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
dnl LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
dnl CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
dnl SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
dnl INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
dnl CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
dnl ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
dnl POSSIBILITY OF SUCH DAMAGE.

dnl Additional macros for `autoconf', some are stolen from C->HS by
dnl Manuel M. T. Chakravarty <chak@is.tsukuba.ac.jp>

dnl ######################################################################
dnl HOPENGL_CHECK_SIZEOF(TYPE [, CROSS-SIZE]) - Compute the size of a type
dnl Adapted from `acgeneral.m4';  Sets a shell variable SIZEOF_<type> 
dnl and calls AC_SUBST on it.
dnl ######################################################################

AC_DEFUN(HOPENGL_CHECK_SIZEOF,
[changequote(<<, >>)dnl
dnl The name to #define.
define(<<HOPENGL_TYPE_NAME>>, translit(sizeof_$1, [a-z *], [A-Z_P]))dnl
dnl The cache variable name.
define(<<HOPENGL_CV_NAME>>, translit(hopengl_cv_sizeof_$1, [ *], [_p]))dnl
changequote([, ])dnl
AC_MSG_CHECKING(size of $1)
AC_CACHE_VAL(HOPENGL_CV_NAME,
[AC_TRY_RUN([#include <stdio.h>
main()
{
  FILE *f=fopen("conftestval", "w");
  if (!f) exit(1);
  fprintf(f, "%d\n", sizeof($1));
  exit(0);
}], HOPENGL_CV_NAME=`cat conftestval`, HOPENGL_CV_NAME=0, ifelse([$2], , , HOPENGL_CV_NAME=$2))])dnl
AC_MSG_RESULT($HOPENGL_CV_NAME)
HOPENGL_TYPE_NAME=$HOPENGL_CV_NAME
AC_SUBST(HOPENGL_TYPE_NAME)dnl
AC_PROVIDE($HOPENGL_TYPE_NAME)
undefine([HOPENGL_TYPE_NAME])dnl
undefine([HOPENGL_CV_NAME])dnl
])

dnl ######################################################################
dnl HOPENGL_CHECK_ALIGNOF(TYPE) - Compute the alignment restriction of a type
dnl
dnl * Adapted from the corresponding test in the Glasgow Haskell Compiler's
dnl   `fptools' configuration written by Simon Marlow(?)
dnl
dnl * This sets the shell variable ALIGNOF_<type> and calls AC_SUBST on it.
dnl
dnl * requires SIZEOF test but AC_CHECK_SIZEOF doesn't call PROVIDE so we 
dnl   can't call REQUIRE)
dnl ######################################################################

AC_DEFUN(HOPENGL_CHECK_ALIGNOF,
[changequote(<<, >>)dnl
dnl The name to #define and to substitute.
define(<<HOPENGL_TYPE_NAME>>, translit(alignof_$1, [a-z *], [A-Z_P]))dnl
dnl The cache variable name.
define(<<HOPENGL_CV_NAME>>, translit(hopengl_cv_alignof_$1, [ *], [_p]))dnl
dnl The name of the corresponding size.
define(<<HOPENGL_SIZEOF_TYPE_NAME>>, translit(sizeof_$1, [a-z *], [A-Z_P]))dnl
dnl The cache variable name of the corresponding size.
define(<<HOPENGL_CV_SIZEOF_NAME>>, translit(hopengl_cv_sizeof_$1, [ *], [_p]))dnl
changequote([, ])dnl
AC_REQUIRE($HOPENGL_SIZEOF_TYPE_NAME)
AC_MSG_CHECKING(alignment of $1)
AC_CACHE_VAL(HOPENGL_CV_NAME,
[AC_TRY_RUN([
#include <stdio.h>
#if HAVE_STDDEF_H
#include <stddef.h>
#endif
#ifndef offsetof
#define offsetof(ty,field) ((size_t)((char *)&((ty *)0)->field - (char *)(ty *)0))
#endif
int
main()
{
  FILE *f=fopen("conftestval", "w");
  if (!f) exit(1);
  fprintf(f, "%d\n", offsetof(struct { char c; $1 ty;},ty));
  exit(0);
}],
HOPENGL_CV_NAME=`cat conftestval`,
HOPENGL_CV_NAME=$HOPENGL_CV_SIZEOF_NAME,
HOPENGL_CV_NAME=$HOPENGL_CV_SIZEOF_NAME)])
AC_MSG_RESULT($HOPENGL_CV_NAME)
HOPENGL_TYPE_NAME=$HOPENGL_CV_NAME
AC_SUBST(HOPENGL_TYPE_NAME)dnl
AC_PROVIDE($HOPENGL_TYPE_NAME)
undefine([HOPENGL_TYPE_NAME])dnl
undefine([HOPENGL_CV_NAME])dnl
])

dnl ######################################################################
dnl SUP_SEARCH_LIBS(INCLUDES, FUNCTION, SEARCH-LIBS [, ACTION-IF-FOUND
dnl            [, ACTION-IF-NOT-FOUND [, OTHER-LIBRARIES]]])
dnl Search for a library defining FUNC, if it's not already available.
dnl This is almost the same as AC_SEARCH_LIBS, but the INCLUDES can be
dnl specified.
dnl ######################################################################

AC_DEFUN(SUP_SEARCH_LIBS,
[AC_PREREQ([2.13])
AC_CACHE_CHECK([for library containing $2], [ac_cv_search_$2],
[ac_func_search_save_LIBS="$LIBS"
ac_cv_search_$2="no"
AC_TRY_LINK([$1], [$2()], [ac_cv_search_$2="none required"])
test "$ac_cv_search_$2" = "no" && for i in $3; do
LIBS="-l$i $6 $ac_func_search_save_LIBS"
AC_TRY_LINK([$1], [$2()],
[ac_cv_search_$2="-l$i"
break])
done
LIBS="$ac_func_search_save_LIBS"])
if test "$ac_cv_search_$2" != "no"; then
  test "$ac_cv_search_$2" = "none required" || LIBS="$ac_cv_search_$2 $LIBS"
  $4
else :
  $5
fi])

dnl ####################### -*- Mode: M4 -*- ###########################
dnl Copyright (C) 98, 1999 Matthew D. Langston <langston@SLAC.Stanford.EDU>
dnl
dnl This file is free software; you can redistribute it and/or modify it
dnl under the terms of the GNU General Public License as published by
dnl the Free Software Foundation; either version 2 of the License, or
dnl (at your option) any later version.
dnl
dnl This file is distributed in the hope that it will be useful, but
dnl WITHOUT ANY WARRANTY; without even the implied warranty of
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
dnl General Public License for more details.
dnl
dnl You should have received a copy of the GNU General Public License
dnl along with this file; if not, write to:
dnl
dnl   Free Software Foundation, Inc.
dnl   Suite 330
dnl   59 Temple Place
dnl   Boston, MA 02111-1307, USA.
dnl ####################################################################


dnl @synopsis MDL_CHECK_LIBM
dnl 
dnl Search for math library (typically -lm).
dnl
dnl The variable LIBM (which is not an output variable by default) is
dnl set to a value which is suitable for use in a Makefile (for example,
dnl in make's LOADLIBES macro) provided you AC_SUBST it first.
dnl
dnl @version 0.01 $Id: aclocal.m4,v 1.15 2003/01/19 18:22:08 panne Exp $
dnl @author Matthew D. Langston <langston@SLAC.Stanford.EDU>

# AC_CHECK_LIBM - check for math library
AC_DEFUN(MDL_CHECK_LIBM,
[AC_REQUIRE([AC_CANONICAL_HOST])dnl
LIBM=
case "$host" in
*-*-beos* | *-*-cygwin*)
  # These system don't have libm
  ;;
*-ncr-sysv4.3*)
  AC_CHECK_LIB(mw, _mwvalidcheckl, LIBM="-lmw")
  AC_CHECK_LIB(m, main, LIBM="$LIBM -lm")
  ;;
*)
  AC_CHECK_LIB(m, main, LIBM="-lm")
  ;;
esac
])

dnl ######################################################################
dnl NOTE: Because of portability issues between different autoconf
dnl versions the AC_HELP_STRING macro has been removed from MDL_HAVE_OPENGL.
dnl Furthermore, caching has been completely rewritten.
dnl ######################################################################

dnl ########################### -*- Mode: M4 -*- #######################
dnl Copyright (C) 98, 1999 Matthew D. Langston <langston@SLAC.Stanford.EDU>
dnl
dnl This file is free software; you can redistribute it and/or modify it
dnl under the terms of the GNU General Public License as published by
dnl the Free Software Foundation; either version 2 of the License, or
dnl (at your option) any later version.
dnl
dnl This file is distributed in the hope that it will be useful, but
dnl WITHOUT ANY WARRANTY; without even the implied warranty of
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
dnl General Public License for more details.
dnl
dnl You should have received a copy of the GNU General Public License
dnl along with this file; if not, write to:
dnl
dnl   Free Software Foundation, Inc.
dnl   Suite 330
dnl   59 Temple Place
dnl   Boston, MA 02111-1307, USA.
dnl ####################################################################


dnl @synopsis MDL_HAVE_OPENGL
dnl 
dnl Search for OpenGL.  We search first for Mesa (a GPL'ed version of
dnl OpenGL) before a vendor's version of OpenGL if we were specifically
dnl asked to with `--with-Mesa=yes' or `--with-Mesa'.
dnl
dnl The four "standard" OpenGL libraries are searched for: "-lGL",
dnl "-lGLU", "-lGLX" (or "-lMesaGL", "-lMesaGLU" as the case may be) and
dnl "-lglut".
dnl
dnl All of the libraries that are found (since "-lglut" or "-lGLX" might
dnl be missing) are added to the shell output variable "GL_LIBS", along
dnl with any other libraries that are necessary to successfully link an
dnl OpenGL application (e.g. the X11 libraries).  Care has been taken to
dnl make sure that all of the libraries in "GL_LIBS" are listed in the
dnl proper order.
dnl
dnl Additionally, the shell output variable "GL_CFLAGS" is set to any
dnl flags (e.g. "-I" flags) that are necessary to successfully compile
dnl an OpenGL application.
dnl
dnl The following shell variable (which are not output variables) are
dnl also set to either "yes" or "no" (depending on which libraries were
dnl found) to help you determine exactly what was found.
dnl
dnl   have_GL
dnl   have_GLU
dnl   have_GLX
dnl   have_glut
dnl
dnl A complete little toy "Automake `make distcheck'" package of how to
dnl use this macro is available at:
dnl
dnl   ftp://ftp.slac.stanford.edu/users/langston/autoconf/ac_opengl-0.01.tar.gz
dnl
dnl Please note that as the ac_opengl macro and the toy example evolves,
dnl the version number increases, so you may have to adjust the above
dnl URL accordingly.
dnl
dnl @version 0.01 $Id: aclocal.m4,v 1.15 2003/01/19 18:22:08 panne Exp $
dnl @author Matthew D. Langston <langston@SLAC.Stanford.EDU>

AC_DEFUN(MDL_HAVE_OPENGL,
[
  AC_REQUIRE([AC_PROG_CC])
  AC_REQUIRE([AC_PATH_X])
  AC_REQUIRE([AC_PATH_XTRA])
  AC_REQUIRE([MDL_CHECK_LIBM])

dnl Check for Mesa first if we were asked to.
  AC_ARG_ENABLE(Mesa,
                AC_HELP_STRING([--enable-Mesa],
                               [Prefer Mesa over a vendor's native OpenGL library (default=no)]),
                use_Mesa=$enableval,
                use_Mesa=no)

  if test x"$use_Mesa" = xyes; then
     GL_search_list="MesaGL  GL  opengl32"
    GLU_search_list="MesaGLU GLU glu32"
    GLX_search_list="MesaGLX GLX"
  else
     GL_search_list="GL  opengl32 MesaGL"
    GLU_search_list="GLU glu32    MesaGLU"
    GLX_search_list="GLX          MesaGLX"
  fi      

  AC_LANG_SAVE
  AC_LANG_C

dnl If we are running under X11 then add in the appropriate libraries.
  if test x"$no_x" != xyes; then
dnl Add everything we need to compile and link X programs to GL_CFLAGS
dnl and GL_X_LIBS.
    GL_CFLAGS="$CPPFLAGS $X_CFLAGS"
    GL_X_LIBS="$X_PRE_LIBS $X_LIBS -lX11 -lXext -lXmu -lXt -lXi $X_EXTRA_LIBS $LIBM"
  fi
  GL_save_CPPFLAGS="$CPPFLAGS"
  CPPFLAGS="$GL_CFLAGS"

  GL_save_LIBS="$LIBS"
  LIBS="$GL_X_LIBS"

  SUP_SEARCH_LIBS([#include <GL/glut.h>], glEnd,         $GL_search_list,  have_GL=yes,   have_GL=no)

  AC_MSG_CHECKING(glTexSubImage1D)
  AC_TRY_LINK([#include <GL/glut.h>], [glTexSubImage1D(GL_TEXTURE_1D,0,0,2,GL_INTENSITY,GL_BYTE,(void*)0)], hopengl_gl_texsubimage1d=yes, hopengl_gl_texsubimage1d=no);
  AC_MSG_RESULT($hopengl_gl_texsubimage1d)

  AC_MSG_CHECKING(glDrawRangeElements)
  AC_TRY_LINK([#include <GL/glut.h>], [glDrawRangeElements(GL_QUADS,0,0,0,GL_UNSIGNED_BYTE,(void*)0)], hopengl_gl_drawrangeelements=yes, hopengl_gl_drawrangeelements=no);
  AC_MSG_RESULT($hopengl_gl_drawrangeelements)

  AC_MSG_CHECKING(glActiveTexture)
  AC_TRY_LINK([#include <GL/glut.h>], [glActiveTexture(GL_TEXTURE1)], hopengl_gl_activetexture=yes, hopengl_gl_activetexture=no);
  AC_MSG_RESULT($hopengl_gl_activetexture)

  AC_MSG_CHECKING(glMultiDrawArrays)
  AC_TRY_LINK([#include <GL/glut.h>], [glMultiDrawArrays(GL_TRIANGLES, (GLint*)0, (GLsizei*)0, 0)], hopengl_gl_multidrawarrays=yes, hopengl_gl_multidrawarrays=no);
  AC_MSG_RESULT($hopengl_gl_multidrawarrays)

  if test x"$hopengl_gl_texsubimage1d" != xyes; then
    hopengl_gl_version=1.0
  else
     if test x"$hopengl_gl_drawrangeelements" != xyes; then
        hopengl_gl_version=1.1
     else
       if test x"$hopengl_gl_activetexture" != xyes; then
          hopengl_gl_version=1.2
       else
         if test x"$hopengl_gl_multidrawarrays" != xyes; then
            hopengl_gl_version=1.3
         else
            hopengl_gl_version=1.4
         fi
       fi
     fi
  fi
  AC_MSG_NOTICE([It looks like GL version ${hopengl_gl_version}])

  AC_MSG_CHECKING(EXT_blend_color)
  AC_TRY_LINK([#include <GL/glut.h>], [glBlendColorEXT((GLclampf)0.0,(GLclampf)0.0,(GLclampf)0.0,(GLclampf)0.0)], hopengl_EXT_blend_color=yes, hopengl_EXT_blend_color=no);
  AC_MSG_RESULT($hopengl_EXT_blend_color)

  AC_MSG_CHECKING(EXT_blend_minmax)
  AC_TRY_LINK([#include <GL/glut.h>], [glBlendEquationEXT(GL_FUNC_ADD_EXT)], hopengl_EXT_blend_minmax=yes, hopengl_EXT_blend_minmax=no);
  AC_MSG_RESULT($hopengl_EXT_blend_minmax)

  AC_MSG_CHECKING(EXT_blend_subtract)
  AC_TRY_LINK([#include <GL/glut.h>], [glBlendEquationEXT(GL_FUNC_SUBTRACT_EXT)], hopengl_EXT_blend_subtract=yes, hopengl_EXT_blend_subtract=no);
  AC_MSG_RESULT($hopengl_EXT_blend_subtract)

  SUP_SEARCH_LIBS([#include <GL/glut.h>], gluNewQuadric, $GLU_search_list, have_GLU=yes,  have_GLU=no)

  AC_MSG_CHECKING(gluGetString)
  AC_TRY_LINK([#include <GL/glut.h>], [gluGetString(GLU_EXTENSIONS)], hopengl_glu_getstring=yes, hopengl_glu_getstring=no);
  AC_MSG_RESULT($hopengl_glu_getstring)

  AC_MSG_CHECKING(gluTessEndPolygon)
  AC_TRY_LINK([#include <GL/glut.h>], [gluTessEndPolygon((GLUtesselator*)0)], hopengl_glu_tessendpolygon=yes, hopengl_glu_tessendpolygon=no);
  AC_MSG_RESULT($hopengl_glu_tessendpolygon)

  AC_MSG_CHECKING(gluUnProject4)
  AC_TRY_LINK([#include <GL/glut.h>], [gluUnProject4(0.0,0.0,0.0,0.0,(GLdouble*)0,(GLdouble*)0,(GLint*)0,0.0,0.0,(GLdouble*)0,(GLdouble*)0,(GLdouble*)0,(GLdouble*)0)], hopengl_glu_unproject4=yes, hopengl_glu_unproject4=no);
  AC_MSG_RESULT($hopengl_glu_unproject4)

  if test x"$hopengl_glu_getstring" != xyes; then
    hopengl_glu_version=1.0
  else
     if test x"$hopengl_glu_tessendpolygon" != xyes; then
        hopengl_glu_version=1.1
     else
       if test x"$hopengl_glu_unproject4" != xyes; then
          hopengl_glu_version=1.2
       else
            hopengl_glu_version=1.3
       fi
     fi
  fi
  AC_MSG_NOTICE([It looks like GLU version ${hopengl_glu_version}])

  SUP_SEARCH_LIBS([#include <GL/glut.h>], glXWaitX,      $GLX_search_list, have_GLX=yes,  have_GLX=no)
  SUP_SEARCH_LIBS([#include <GL/glut.h>], glutMainLoop,  glut32 glut,      have_glut=yes, have_glut=no)

  if test -n "$LIBS"; then
    GL_LIBS="$LDFLAGS $LIBS"
  else
    GL_LIBS="$LDFLAGS"
    GL_CFLAGS=
  fi

  AC_CACHE_CHECK([OpenGL flags], mdl_cv_gl_cflags, [mdl_cv_gl_cflags="$GL_CFLAGS"])
  GL_CFLAGS="$mdl_cv_gl_cflags"
  AC_SUBST(GL_CFLAGS)
  AC_CACHE_CHECK([OpenGL libs],  mdl_cv_gl_libs,   [mdl_cv_gl_libs="$GL_LIBS"])
  GL_LIBS="$mdl_cv_gl_libs"
  AC_SUBST(GL_LIBS)

dnl Reset GL_X_LIBS regardless, since it was just a temporary variable
dnl and we don't want to be global namespace polluters.
  GL_X_LIBS=

  LIBS="$GL_save_LIBS"
  CPPFLAGS="$GL_save_CPPFLAGS"

  AC_LANG_RESTORE
])
